using iFileProxy.Models;
using iFileProxy.Services;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.Filters;

namespace iFileProxy.Attributes
{
    [AttributeUsage(AttributeTargets.Class | AttributeTargets.Method)]
    public class AuthorizeAttribute : Attribute, IAuthorizationFilter
    {
        private readonly UserMask[] _allowedMasks;

        public AuthorizeAttribute(params UserMask[] allowedMasks)
        {
            _allowedMasks = allowedMasks;
        }

        public void OnAuthorization(AuthorizationFilterContext context)
        {
            var userId = context.HttpContext.Items["User"]?.ToString();
            if (string.IsNullOrEmpty(userId))
            {
                context.Result = new JsonResult(new CommonRsp { Message = "未授权", Retcode = -1 })
                {
                    StatusCode = StatusCodes.Status401Unauthorized
                };
                return;
            }

            if (_allowedMasks.Length > 0)
            {
                var dbGateService = context.HttpContext.RequestServices.GetRequiredService<DatabaseGateService>();
                var user = dbGateService.GetUserByIdAsync(userId).Result;
                
                if (user == null || !_allowedMasks.Contains(user.Mask) )
                {
                    context.Result = new JsonResult(new CommonRsp  { Message = "权限不足" ,Retcode = 1})
                    {
                        StatusCode = StatusCodes.Status403Forbidden
                    };
                }
            }
        }
    }
} 