﻿using iFileProxy.Helpers;
using iFileProxy.Services;
using iFileProxy.Models;
using iFileProxy.Config;
using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json;
using Serilog;

namespace iFileProxy.Controllers
{
    /// <summary>
    /// 访客信息相关
    /// </summary>
    [Route("/[controller]")]
    [ApiController]
    public class VisitorManagementController : ControllerBase
    {
        private readonly DatabaseGateService _dbGateService;
        public VisitorManagementController(DatabaseGateService dbGate)
        {
            _dbGateService = dbGate;
        }
        private readonly static Serilog.ILogger _logger = Log.Logger.ForContext<VisitorManagementController>();


        [HttpDelete("DeleteInfo")]
        public ActionResult<CommonRsp> DeleteInfo()
        {
            string ipAddr = MasterHelper.GetClientIPAddr(HttpContext);
            var d = JsonConvert.DeserializeObject<List<TaskInfo>>(_dbGateService.GetTaskListByStateAndIp(ipAddr, TaskState.Running));
            int rd = (d != null) ? d.Count : 0;
            if (rd <= 0)
            {
                d = JsonConvert.DeserializeObject<List<TaskInfo>>(_dbGateService.GetTaskListByIP(ipAddr));
                foreach (var taskInfo in d)
                {
                    var dep = _dbGateService.CheckCacheDependencies(taskInfo.TaskId,ipAddr);
                    if (dep.Count <= 0)
                    {
                        if (MasterHelper.CheckDownloadFileExists(taskInfo.FileName))
                        {
                            MasterHelper.RemoveDownloadFile(taskInfo.FileName);
                            _logger.Information("[客户端主动申请删除信息数据] 正在删除文件: {f}", taskInfo.FileName);
                        }
                    }
                    else
                        _logger.Warning($"准备删除的文件正在被以下Task所依赖: {JsonConvert.SerializeObject(dep)} 文件将不会被删除");

                }
                return Ok(new CommonRsp() { Retcode = 0, Message = "succ", Data = _dbGateService.DeleteTaskInfoByIpAddr(ipAddr)});
            }
            else
                return Ok(new CommonRsp() {Retcode = -1, Message = "你还有正在运行的任务, 你现在不能删除你的数据, 请稍后等待任务完成后重试", Data = rd });
        }
    }
}
