﻿using Microsoft.AspNetCore.Components;

namespace iFileProxy
{
    using iFileProxy.Helpers;
    using iFileProxy.Models;
    using Microsoft.AspNetCore.Http;
    using Serilog;
    using System;
    using System.Net;
    using System.Text.Json;
    using System.Threading.Tasks;

    public class ErrorHandlerMiddleware
    {
        private readonly RequestDelegate _next;

        private readonly static Serilog.ILogger _logger = Log.Logger.ForContext<ErrorHandlerMiddleware>();


        public ErrorHandlerMiddleware(RequestDelegate next)
        {
            _next = next;
        }

        public async Task InvokeAsync(HttpContext context)
        {
            try
            {
                await _next(context);
                if (context.Response.StatusCode == 404)
                { 
                    context.Response.ContentType = "application/json";
                    await context.Response.WriteAsync( JsonSerializer.Serialize(new CommonRsp { Retcode = 404, Message = "this route not exists." }));

                }
            }
            catch (Exception ex)
            {
                await HandleExceptionAsync(context, ex);
            }
        }

        private static Task HandleExceptionAsync(HttpContext context, Exception exception)
        {
            var code = HttpStatusCode.InternalServerError; // 500 if unexpected
            _logger.Error("Crash Data: {exception}\nContext: {context}", exception,  JsonSerializer.Serialize (MasterHelper.ExtractDebugInfo(context)));
            switch (exception)
            {
                
                case NotImplementedException:
                    code = HttpStatusCode.NotImplemented; // 501
                    break;
                case UnauthorizedAccessException:
                    code = HttpStatusCode.Unauthorized; // 401
                    break;
                case ArgumentException:
                    code = HttpStatusCode.BadRequest; // 400
                    break;
                    // Add more cases for different types of exceptions
            }

            context.Response.ContentType = "application/json";
            context.Response.StatusCode = (int)code;

            return context.Response.WriteAsync(JsonSerializer.Serialize(new CommonRsp { Retcode = 1, Message = "server internal error" }));
        }
    }
}
