using System.Text.Json;

namespace iFileProxy.Models
{
    public class HttpContextDebugInfo
    {
        /// <summary>
        /// 请求ID
        /// </summary>
        public string RequestId { get; set; } = string.Empty;

        /// <summary>
        /// 请求路径
        /// </summary>
        public string Path { get; set; } = string.Empty;

        /// <summary>
        /// 请求方法
        /// </summary>
        public string Method { get; set; } = string.Empty;

        /// <summary>
        /// 客户端IP
        /// </summary>
        public string ClientIP { get; set; } = string.Empty;

        /// <summary>
        /// User-Agent
        /// </summary>
        public string UserAgent { get; set; } = string.Empty;

        /// <summary>
        /// 请求头信息
        /// </summary>
        public Dictionary<string, string[]> Headers { get; set; } = [];

        /// <summary>
        /// 查询参数
        /// </summary>
        public Dictionary<string, string[]> QueryParams { get; set; } = [];

        /// <summary>
        /// 表单数据
        /// </summary>
        public Dictionary<string, string[]> FormData { get; set; } = [];

        /// <summary>
        /// Cookie信息
        /// </summary>
        public Dictionary<string, string> Cookies { get; set; } = [];

        /// <summary>
        /// 请求体内容
        /// </summary>
        public string? RequestBody { get; set; }

        /// <summary>
        /// 请求时间
        /// </summary>
        public DateTime RequestTime { get; set; } = DateTime.Now;

        /// <summary>
        /// 服务器主机名
        /// </summary>
        public string Host { get; set; } = string.Empty;

        /// <summary>
        /// 请求内容类型
        /// </summary>
        public string ContentType { get; set; } = string.Empty;

        /// <summary>
        /// 内容长度
        /// </summary>
        public long? ContentLength { get; set; }

        /// <summary>
        /// 是否是HTTPS
        /// </summary>
        public bool IsHttps { get; set; }

        /// <summary>
        /// 转换为格式化的字符串
        /// </summary>
        public override string ToString()
        {
            return JsonSerializer.Serialize(this, new JsonSerializerOptions 
            { 
                WriteIndented = true 
            });
        }
    }
} 