using System.Diagnostics;
using ThreadState = System.Diagnostics.ThreadState;

namespace iFileProxy.Models
{
    /// <summary>
    /// 进程详细信息模型
    /// </summary>
    public class ProcessInfo
    {
        /// <summary>
        /// 进程ID
        /// </summary>
        public int ProcessId { get; set; }

        /// <summary>
        /// 进程名称
        /// </summary>
        public string ProcessName { get; set; } = string.Empty;

        /// <summary>
        /// 启动时间
        /// </summary>
        public DateTime StartTime { get; set; }

        /// <summary>
        /// 运行时长（秒）
        /// </summary>
        public double RunningTime { get; set; }

        /// <summary>
        /// 线程数
        /// </summary>
        public int ThreadCount { get; set; }

        /// <summary>
        /// 命令行参数
        /// </summary>
        public string CommandLine { get; set; } = string.Empty;

        /// <summary>
        /// 进程优先级
        /// </summary>
        public ProcessPriorityClass PriorityClass { get; set; }

        /// <summary>
        /// 工作目录
        /// </summary>
        public string WorkingDirectory { get; set; } = string.Empty;

        /// <summary>
        /// 线程信息列表
        /// </summary>
        public List<ThreadInfo> Threads { get; set; } = new();
    }

    /// <summary>
    /// 线程信息模型
    /// </summary>
    public class ThreadInfo
    {
        /// <summary>
        /// 线程ID
        /// </summary>
        public int ThreadId { get; set; }

        /// <summary>
        /// 线程状态
        /// </summary>
        public ThreadState ThreadState { get; set; }

        /// <summary>
        /// 线程优先级
        /// </summary>
        public ThreadPriorityLevel Priority { get; set; }
    }
} 