using Serilog.Core;
using Serilog.Events;
using iFileProxy.Services;
using Newtonsoft.Json;

namespace iFileProxy.Sinks
{
    public class DatabaseLogSink : ILogEventSink
    {
        private readonly DatabaseGateService _dbGateService;
        private readonly LogEventLevel _restrictedToMinimumLevel;

        public DatabaseLogSink(DatabaseGateService dbGateService, LogEventLevel restrictedToMinimumLevel)
        {
            _dbGateService = dbGateService;
            _restrictedToMinimumLevel = restrictedToMinimumLevel;
        }

        public void Emit(LogEvent logEvent)
        {
            if (logEvent.Level < _restrictedToMinimumLevel) return;

            try
            {
                var log = new SystemLog
                {
                    Level = logEvent.Level.ToString(),
                    Message = logEvent.RenderMessage(),
                    Exception = logEvent.Exception?.ToString(),
                    Properties = JsonConvert.SerializeObject(logEvent.Properties.ToDictionary(
                        kvp => kvp.Key,
                        kvp => kvp.Value.ToString()
                    )),
                    Timestamp = logEvent.Timestamp.DateTime
                };

                // 异步写入数据库
                Task.Run(async () => await _dbGateService.CreateSystemLogAsync(log));
            }
            catch
            {
                // 记录日志失败时不抛出异常，避免影响应用程序运行
            }
        }
    }
} 